//---------------------------------------------------------------------------

#include <vcl.h>
#include <stdlib.h>
#include <stdio.h>
#pragma hdrstop

#include "Unit1.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;



void wordwr(unsigned char *data,int val)
{
	*data++=(val>>8)&0xff;
	*data++=(val&0xff);
}



unsigned char rom48[16384];
unsigned char ay[1024*1024];
int ay_size;
int ay_block_cnt;
int ay_data_off;



void ay_create(int start,unsigned char *title,unsigned char *author)
{
	int i;

	memcpy(ay,"ZXAYEMUL",8);
	ay[8]=1;//file version
	ay[9]=3;//player version (3 for 48K support)
	ay[10]=0;//special player, short int
	ay[11]=0;
	wordwr(&ay[12],140-12);//author offset
	wordwr(&ay[14],173-14);//misc offset
	ay[16]=0;//number of songs in file-1
	ay[17]=0;//first song-1
	wordwr(&ay[18],2);//end struct offset
	wordwr(&ay[20],173-20);//NTStr offset
	wordwr(&ay[22],2);//song header offset
	ay[24]=0;//AY A
	ay[25]=1;//AY B
	ay[26]=2;//AY C
	ay[27]=3;//AY Noise
	wordwr(&ay[28],0);//song length (0 for unknown)
	wordwr(&ay[30],0);//fade length
	ay[32]=0;//hireg
	ay[33]=0;//loreg
	wordwr(&ay[34],4);//points offset
	wordwr(&ay[36],8);//data offset
	wordwr(&ay[38],start-1);//stack
	wordwr(&ay[40],start);//init
	wordwr(&ay[42],0x0038);//interrupt
	for(i=0;i<16;i++)
	{
		wordwr(&ay[44+i*6],0);//addr for z80
		wordwr(&ay[46+i*6],0);//length
		wordwr(&ay[48+i*6],0);//offset in file
	}
	wordwr(&ay[44+16*6],0);//end of blocks list
	memcpy(&ay[140],title,32);//title
	ay[172]=0;
	memcpy(&ay[173],author,32);//author
	ay[205]=0;

	ay_block_cnt=0;
	ay_data_off=206;
	ay_size=ay_data_off;
}



void ay_block(unsigned char *data,int addr,int size)//16  
{
	int off;

	off=44+ay_block_cnt*6;

	wordwr(&ay[off+0],addr);//addr of block in Z80
	wordwr(&ay[off+2],size);//length of block
	wordwr(&ay[off+4],ay_data_off-(off+4));//offset in file
	memcpy(ay+ay_data_off,data,size);

	ay_block_cnt++;
	ay_size+=size;
	ay_data_off+=size;
}



void ay_write(const char *filename)
{
	FILE *file;

	file=fopen(filename,"wb");

	if(file)
	{
		fwrite(ay,ay_size,1,file);
		fclose(file);
	}
}



void __fastcall TForm1::EnableConv(bool e)
{
	ButtonSave->Enabled=e;
}


bool __fastcall TForm1::Convert(AnsiString sfile,AnsiString dfile)
{
	FILE *file;
	unsigned char *tap;
	int tsize;
	int coff,csize,start;
	int pp,off1,off2;

	file=fopen(sfile.c_str(),"rb");
	if(!file) return false;
	fseek(file,0,SEEK_END);
	tsize=ftell(file);
	fseek(file,0,SEEK_SET);
	tap=(unsigned char*)malloc(tsize);
	fread(tap,tsize,1,file);
	fclose(file);

	coff=2+19+(tap[14]+tap[15]*256)+4;
	csize=tap[coff+14]+tap[coff+15]*256;
	start=tap[coff+16]+tap[coff+17]*256;
	coff=coff+2+19+3;

	pp=16*8;
	off1=0;
	off2=0;

	while(pp<tsize)
	{
		if(memcmp(&tap[pp],"Title:",6)==0)
		{
			off1=pp-1;
			pp+=7;
			break;
		}
		pp++;
	}

	while(pp<tsize)
	{
		if(memcmp(&tap[pp],"Author:",7)==0)
		{
			off2=pp;
			pp+=8;
			break;
		}
		pp++;
	}

	if(!off1||!off2)
	{
		free(tap);
		return false;
	}

	ay_create(start,&tap[off1],&tap[off2]);
	ay_block(&tap[coff],start,csize);
	if(tap[coff]==0xf3&&tap[coff+1]==0xd9&&tap[coff+2]==0xe5)//Special FX engine, needs some ROM data for drums
	{
		ay_block(&rom48[0x0100],0x0100,10);
		ay_block(&rom48[0x005a],0x005a,0x02ae-0x005a);
		ay_block(&rom48[0x0f18],0x0f18,20);
	}
	if(tap[coff]==0x21&&tap[coff+1]==0x80&&tap[coff+2]==0xed)//Phaser1 synth drums, needs some ROM data for noise drums
	{
		ay_block(&rom48[2480],2480,2480);
		ay_block(&rom48[1070],1070,1070);
		ay_block(&rom48[365],365,365);
	}
	ay_write(dfile.c_str());

	free(tap);

	return true;
}


//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
: TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TForm1::ButtonOpenClick(TObject *Sender)
{
	bool enable;
	int i;

	enable=false;

	if(OpenDialog1->Execute())
	{
		for(i=0;i<OpenDialog1->Files->Count;i++)
		{
			if(FileExists(OpenDialog1->Files->Strings[i]))
			{
				enable=true;
				ListBoxFiles->Items->Add(OpenDialog1->Files->Strings[i]);
			}
		}
	}
	EnableConv(enable);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::ButtonSaveClick(TObject *Sender)
{
	AnsiString name;
	int i;

	for(i=0;i<ListBoxFiles->Items->Count;i++)
	{
		ListBoxFiles->ItemIndex=i;
		name=ListBoxFiles->Items->Strings[i];
		name=name.SubString(0,name.LastDelimiter(".")-1);
		name=name+".ay";

		if(FileExists(name))
		{
			if(Application->MessageBox("File exists. Overwrite?","Confirm",MB_YESNO)==IDNO) continue;
		}
		Convert(ListBoxFiles->Items->Strings[i],name);
	}

	Application->MessageBox("Done!","Confirm",MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormCreate(TObject *Sender)
{
	FILE *file;

	file=fopen("48.rom","rb");

	if(!file)
	{
		Application->MessageBox("You need 48.rom file in the program directory in order to use this tool.","Error",MB_OK);
		exit(-1);
	}
	else
	{
		fread(rom48,16384,1,file);
		fclose(file);
	}
}
//---------------------------------------------------------------------------

